/**
 * @desc: index
 */

const _fs = require('fs');
const _fse = require('fs-extra');
const _path = require('path');
const _webpack = require('webpack');
const _chalk = require('chalk');
const _hash = require('object-hash');
const _stats = require('../../webpack-base/stats');
const _entry = require('../../webpack-dll/entry');
const _env = require('../../config/env');
const _dirs = require('../../config/dir-vars').dirs;
const _check = require('../../config/env-util').check;

// eslint-disable-next-line max-len
module.exports = ({ env, config }) => (step => step[1]())((({ vh, releasePath, createHash, force }) => ({
  
  // check whether env is correct
  1() {
    env && _check(env) ? this.go(2) : this.go(6, `The unknown process.env.NODE_ENV: ${env}`, true);
  },
  
  // check whether release.json file is exist, if not, step 4; otherwise, step 3
  2() {
    _fs.stat(releasePath, error => {
      if (error) {
        error.code === 'ENOENT' ? this.go(4, null, vh(), createHash(config)) : this.go(6, error.toString(), true);
      } else {
        this.go(3);
      }
    });
  },
  
  // read release.json file to check whether need to rebuild dll
  // if it need to rebuild, step 4; otherwise, step end
  3() {
    _fs.readFile(releasePath, 'utf-8', (error, data) => {
      error && this.go(6, error.toString(), true);
      data = JSON.parse(data);
      const versionHash = vh();
      const configHash = createHash(config);
      if (data[env] && data[env].version === versionHash && data[env].config === configHash) {
        force ? this.go(4, data, versionHash, configHash) : this.end();
      } else {
        this.go(4, data, versionHash, configHash);
      }
    });
  },
  
  // create or update release.json file
  4(data, versionHash, configHash) {
    data = Object.assign({}, data, { [env]: { version: versionHash, config: configHash } });
    _fse.outputJson(releasePath, data, error => {
      error ? this.go(6, error.toString(), true) : this.go(5);
    });
  },
  
  // run webpack dll config
  5() {
    _webpack(config, (error, stats) => {
      if (error) {
        this.go(6, `step 5: webpack dll compile failed !${error.toString()}`, true);
      } else {
        this.end();
        console.log(stats.toString(_stats));
      }
    });
  },
  
  // show error message
  6(msg, exit) {
    console.error(_chalk.red(msg));
    exit ? process.exit(1) : this.end();
  },
  
  // step end
  end() {},
  
  // step go
  go(number, ...args) {
    this[number](...args);
  }
  
}))({
  
  // create hash of webpack dll entry packages
  vh: () => _hash(((fn, gv) => fn(Object.keys(_entry)
  .map(key => _entry[key]).join().split(',')
  .filter((item, index, array) => array.indexOf(item, index + 1) === -1), gv))((pl, gv) => {
    const obj = {};
    pl.forEach(pkg => { obj[pkg] = gv(pkg); });
    return obj;
  }, pkg => {
    try {
      return require(`${pkg}/package.json`).version;
    } catch (e) {
      return '0.0.0';
    }
  })),
  
  // release.json file absolute path
  releasePath: _path.join(_dirs.dll, _env.dll.release),
  
  createHash: _hash,
  
  // if has "--force" option, it will rebuild every time even if it has checked that
  // webpack dll doesn't need to rebuild
  force: process.argv.includes('--force')
  
}));
