/**
 * @desc: directory structure configuration, export directory name
 * which named as vars&_vars and directory path which named as dirs&_dirs
 */

const _path = require('path');

const [_node, _src, _build] = ['node_modules', 'src', 'dist'];

const _variables = { // 文件名称
  
  node: _node, // nodejs目录
  
  dll: `.dll->${_node}`,
  
  src: _src, // 开发目录
  
  build: _build, // 文件生成存放目录
  
  launcher: 'launcher', // 配置目录
  
  data: `data->${_src}`, // 可存放模拟数据json
  
  fonts: `fonts->${_src}`, // 存放字体文件
  
  images: `images->${_src}`, // 存放公用图片
  
  styles: `styles->${_src}`, // 存放公用css样式文件
  
  scripts: `scripts->${_src}`, // 存放js文件
  
  views: `views->${_src}`, // 存放vue页面文件
  
  comps: `components->${_src}`, // 存放vue公用组件
  
  deploy: `deploy->${_src}`, // 引导配置项（不知道写啥）
  
  filters: `filters->${_src}`, // 存放自定义公用vue过滤器
  
  directives: `directives->${_src}`, // 存放自定义公用vue指令
  
  store: `store->${_src}`, // 存放vuex状态
  
  mixins: `mixins->${_src}`, // 存放公用mixins
  
  services: `services->${_src}`, // 存放api接口
  
  plugins: `plugins->${_src}`, // 存放开源插件文件夹名称（一般开源插件通过npm安装即可，所以不建议在该文件夹下存放文件，除了特殊情况）
  
  routes: `routes->${_src}`, // vue路由页面配置
  
  utils: `utils->${_src}`, // 存放公共工具方法
  
  ui: `ui->${_src}` // 存放基础组件
  
};

((vars, dirs) => {
  module.exports = { vars, dirs };
})((() => {
  const vars = {};
  const map = value => value.replace(/->.+/g, '');
  Object.keys(_variables).forEach(key => { vars[key] = map(_variables[key]); });
  return vars;
})(), (() => {
  const dirs = {};
  const varToDir = variable => fn => fn(variable.split('->'));
  Object.keys(_variables).forEach(key => {
    dirs[key] = varToDir(_variables[key])(x => _path.join(__dirname, '../../', x[1] || '', x[0]));
  });
  return dirs;
})());
