/**
 * @desc: webpack base config
 */

const _path = require('path');
const _webpack = require('webpack');
const _htmlWebpackPlugin = require('html-webpack-plugin');
const _dirs = require('../config/dir-vars').dirs;

/**
 * @param options {Object}
 * {
 *   htmlMinify {Boolean|Object}: html minify
 * }
 */
module.exports = (options = {
  
  htmlMinify: false
  
}) => ({
  
  entry: {
    app: _path.join(_dirs.src, 'main.js')
  },
  
  output: {
    path: _dirs.build,
    publicPath: '/'
  },
  
  resolve: {
    modules: [_dirs.node, _dirs.src],
    alias: require('./alias'),
    extensions: ['.js', '.vue', '.json']
  },
  
  stats: require('./stats'),
  
  cache: {},
  
  plugins: [
    new _webpack.ProvidePlugin({
      Vue: 'vue',
      _camelCase: 'camel-case'
    }),
    new _htmlWebpackPlugin({
      minify: options.htmlMinify,
      favicon: _path.join(_dirs.src, 'favicon.ico'),
      template: _path.join(_dirs.src, 'index.html'),
      filename: _path.join(_dirs.build, 'index.html')
    })
  ]
  
});
