/**
 * @desc: webpack module's base rules config
 */

const _path = require('path');
const { dirs: _dirs, vars: _vars } = require('../config/dir-vars');

const _happypackLoader = 'happypack/loader';
const _element = _path.join(_dirs.node, 'element-ui', 'lib', 'theme-default');

/**
 * @param options {Object}
 * {
 *   eslint {Object}: eslint-loader option
 *   hash {Boolean}: webpack hash, not use in development env(recommended)
 *   extractCss {Object|Boolean}
 * }
 */
module.exports = (options = {}) => ({
  
  module: {
    rules: [
      {
        test: /\.(vue|js|html)$/,
        loader: 'eslint-loader',
        enforce: 'pre',
        include: [_dirs.src],
        options: Object.assign({ cache: true }, options.eslint || {})
      },
      {
        test: /\.vue$/,
        loader: `${_happypackLoader}?id=vue`,
        include: [_dirs.src]
      },
      {
        test: /\.js$/,
        loader: `${_happypackLoader}?id=babel`,
        include: [_dirs.src, _path.join(_dirs.launcher, 'webpack-dev', 'hot-client.js')]
      },
      {
        test: /\.html$/,
        loader: 'html-loader',
        include: [_path.join(_dirs.src, 'index.html')]
      },
      {
        test: /\.(svg|woff2?|ttf|eot)\??.*$/,
        loader: 'url-loader',
        include: [_dirs.fonts, _element],
        options: {
          limit: 10000,
          name: `${_vars.fonts}/[name]${options.hash ? '.[hash:6]' : ''}.[ext]`
        }
      },
      {
        test: /\.(png|jpe?g|gif|svg|ico)(\?.*)?$/,
        loader: 'url-loader',
        include: [_dirs.images, _dirs.views, _dirs.comps, _dirs.ui],
        options: {
          limit: 10000,
          name: options.hash ? `${_vars.images}/[name].[hash:6].[ext]` : '[path][name].[ext]'
        }
      },
      {
        test: /\.css$/,
        use: options.extractCss || ['vue-style-loader', 'css-loader', 'postcss-loader'],
        include: [_dirs.styles, _dirs.views, _element]
      }
    ]
  }
  
});
