/**
 * @desc: webpack plugin config for all environments
 */

const _path = require('path');
const _webpack = require('webpack');
const _happyPack = require('happypack');
const _os = require('os');
const _hash = require('object-hash');
const _hardSourceWebpackPlugin = require('hard-source-webpack-plugin');
const _dirs = require('../config/dir-vars').dirs;

const _cache = _path.join(_dirs.node, '.cache/hard-source', '[confighash]');

const _happyPackConfig = () => ({
  threadPool: _happyPack.ThreadPool({ size: _os.cpus().length })
});

/**
 * @param options {Object}
 * {
 *   vue {Object}: vue-loader config options
 *   hardSource {Boolean}: whether use hard-source-webpack-plugin,
 *   not use in development env(recommended)
 * }
 */
module.exports = (options = {
  
  vue: {}
  
}) => ({
  
  plugins: [
    new _webpack.ProgressPlugin(),
    new _happyPack(Object.assign(_happyPackConfig(), {
      id: 'babel',
      loaders: [
        {
          path: 'babel-loader',
          query: { cacheDirectory: true }
        }
      ]
    })),
    new _happyPack(Object.assign(_happyPackConfig(), {
      id: 'vue',
      loaders: [
        {
          path: 'vue-loader',
          query: Object.assign({ esModule: false }, options.vue)
        }
      ]
    }))
  ].concat(options.hardSource ? [
    new _hardSourceWebpackPlugin({
      cacheDirectory: _cache,
      recordsPath: _path.join(_cache, 'records.json'),
      configHash: webpackConfig => _hash(webpackConfig),
      environmentHash: () => _hash(require('../../package.json'))
    })
  ] : [])
  
});
