/**
 * @desc: build a local development server
 */

const _express = require('express');
const _webpack = require('webpack');
const _open = require('opn');
const _chalk = require('chalk');
const _proxy = require('http-proxy-middleware');
const _webpackConfig = require('./index');
const _dirs = require('../config/dir-vars').dirs;
const _dev = require('../config/env').dev;

const _app = _express();
const _compiler = _webpack(_webpackConfig);
const _hotMiddleware = require('webpack-hot-middleware')(_compiler);
const _devMiddleware = require('webpack-dev-middleware')(_compiler, {
  quiet: _dev.quiet,
  publicPath: _webpackConfig.output.publicPath,
  stats: _webpackConfig.stats
});

let _resolve;
const _promise = new Promise(r => { _resolve = r; });

_compiler.plugin('compilation', compilation => {
  compilation.plugin('html-webpack-plugin-after-emit', (data, cb) => {
    _hotMiddleware.publish({ action: 'reload' });
    cb();
  });
});

Object.keys(_dev.proxy).forEach(key => {
  let options = _dev.proxy[key];
  typeof options === 'string' && (options = { target: options });
  _app.use(_proxy(options.filter || key, options));
});

// eslint-disable-next-line import/newline-after-import
_app.use(require('connect-history-api-fallback')());
_app.use(_hotMiddleware);
_app.use(_devMiddleware);
_app.use('/data', _express.static(_dirs.data));

_devMiddleware.waitUntilValid(() => {
  const url = `http://${require('ip').address()}:${_dev.port}`;
  _dev.open ? _open(url) : console.log(_chalk.green(`listening at ${url}\n`));
  _resolve();
});

const _server = _app.listen(_dev.port, error => {
  error && console.log(_chalk.red(error));
});

module.exports = {
  ready: _promise,
  close: () => { _server.close(); }
};
