/**
 * @desc: webpack dll config
 */

const _webpack = require('webpack');
const _entry = require('./entry');
const _dirs = require('../config/dir-vars').dirs;

const _library = '[name]_[chunkhash:6]';
const _regExp = Object.keys(_entry).map(key => new RegExp(_entry[key].join('|')));
const _happypackLoader = 'happypack/loader';

/**
 * @param options {Object}
 * {
 *   extractCss {Object|Boolean}
 *   manifestPath(required) {String}: manifest json file's absolute path
 * }
 */
module.exports = (options = {}) => ({
  
  entry: _entry,
  
  output: {
    library: _library
  },
  
  module: {
    rules: [
      {
        test: /\.js$/,
        loader: `${_happypackLoader}?id=babel`,
        exclude: [_dirs.node]
      },
      {
        test: /\.css$/,
        use: options.extractCss || ['vue-style-loader', 'css-loader', 'postcss-loader'],
        include: _regExp
      }
    ]
  },
  
  resolve: {
    alias: require('../webpack-base/alias')
  },
  
  stats: require('../webpack-base/stats'),
  
  plugins: [
    new _webpack.DllPlugin({
      name: _library,
      path: options.manifestPath
    })
  ]
  
});
