/**
 * @desc: webpack dll reference config which use webpack.DllReferencePlugin
 */

const _path = require('path');
const _webpack = require('webpack');
const _entry = require('../webpack-dll/entry');
const _addAssetHtmlPlugin = require('add-asset-html-webpack-plugin');
const { dirs: _dirs, vars: _vars } = require('../config/dir-vars');

/**
 * @param options {Object}
 * {
 *   dll(required) {Object}: config/env.js->dev.dll/prod.dll
 *   hash {Boolean}: Add hash to js bundle file
 *   sourceMap: {Boolean}: Create js bundle sourcemap file
 * }
 */
module.exports = (options = {
  
  hash: true
  
}) => ({
  
  plugins: [
    new _addAssetHtmlPlugin(Object.keys(_entry).map(key => ({
      filepath: _path.join(_dirs.dll, `${key}${options.dll.nameStyle}.js`),
      hash: options.hash,
      publicPath: `/${_vars.scripts}`,
      outputPath: _vars.scripts,
      includeSourcemap: !!options.sourceMap
    }))),
    new _webpack.DllReferencePlugin({
      manifest: require(`${_dirs.dll}/${options.dll.manifest}`)
    })
  ]
  
});
