/**
 * @desc: common config which used by webpack and dll at production environment
 */

const _path = require('path');
const _webpack = require('webpack');
const _extractTextPlugin = require('extract-text-webpack-plugin');
const _optimizeCssAssetsPlugin = require('optimize-css-assets-webpack-plugin');
const _parallelUglifyPlugin = require('webpack-parallel-uglify-plugin');
const _prod = require('../config/env').prod;
const { dirs: _dirs, vars: _vars } = require('../config/dir-vars');

module.exports = () => ({
  
  devtool: _prod.sourceMap ? '#source-map' : false,
  
  plugins: [
    new _webpack.DefinePlugin({
      'process.env': { NODE_ENV: JSON.stringify(_prod.env) }
    }),
    new _webpack.HashedModuleIdsPlugin(),
    
    // solve extract-text-webpack-plugin css duplication problem
    // set zindex false because cssnano unsafe changes the zindex automatically
    // get more info at http://cssnano.co/optimisations/
    new _optimizeCssAssetsPlugin({
      cssProcessorOptions: { autoprefixer: false, safe: true }
    }),
    // not use webpack.optimize.UglifyJsPlugin
    // webpack-parallel-uglify-plugin can improve the speed of webpack compile
    new _parallelUglifyPlugin({
      cacheDir: _path.join(_dirs.node, '.cache', 'uglify'),
      uglifyJS: _prod.uglifyJS
    }),
    new _extractTextPlugin(`${_vars.styles}/[name].bundle.[contenthash:6].css`),
    new _webpack.optimize.ModuleConcatenationPlugin()
  ]
  
});
