/**
 * @desc: webpack config in production environment
 */

const _merge = require('webpack-merge');
const _baseConfig = require('../webpack-base');
const _baseLoaders = require('../webpack-base/loaders');
const _basePlugins = require('../webpack-base/plugins');
const _prodBaseConfig = require('../webpack-prod/base');
const _dllReferenceConfig = require('../webpack-dll/reference');
// const _inlineChunkPlugin = require('html-webpack-inline-chunk-plugin');
const _extractCss = require('./extract-css');
const _prod = require('../config/env').prod;
const _vars = require('../config/dir-vars').vars;

module.exports = _merge(_baseConfig({
  
  htmlMinify: _prod.htmlMinify
  
}), _baseLoaders({
  
  eslint: { failOnError: true },
  
  hash: true,
  
  extractCss: _extractCss
  
}), _basePlugins({
  
  vue: { loaders: { css: _extractCss } },
  
  hardSource: true
  
}), _prodBaseConfig(), _dllReferenceConfig({
  
  dll: _prod.dll,
  
  hash: true,
  
  sourceMap: _prod.sourceMap
  
}), {
  
  output: {
    filename: `${_vars.scripts}/[name].bundle.[chunkhash:6].js`,
    chunkFilename: `${_vars.scripts}/[name].[chunkhash:6].js`
  },
  
  // recordsPath: _path.join(_dirs.launcher, 'webpack-prod', _prod.records),
  
  stats: _prod.quiet ? 'none' : _baseConfig.stats,
  
  plugins: [
    // not inject manifest file to html,
    // create script tag and inline manifest code to html
    // new _inlineChunkPlugin({ inlineChunks: ['manifest'] })
  ]
  
});
