/**
 * @desc: index
 */

import cToast from './c-toast';
import PopupManager from '../utils/popupManager';


/**
 * 用法： this.$toast已注册为全局方法， 所以可以在组件中直接调用
 *
 *  this.$toast(msg) || this.$toast({msg, duration})
 *
 * @param options
 * {
 *   msg: 显示的信息
 *   duration: ms -- 一般不需要设置,会自动根据信息长度计算显示时间
 *   posBottom: {string} '100px' | '50%'
 * }
 *
 */

const popupManager = new PopupManager(5);

const ToastCtr = Vue.extend(cToast);

function mountIns(ins) {
  ins.vm = ins.$mount();
  window.document.body.appendChild(ins.vm.$el);
  ins.vm.visible = true;
  ins.dom = ins.vm.$el;
  ins.dom.style.zIndex = 100;
  /**
   * setTimeout make UI thread has chance to work
   * which guarantee former Toast Instance has been
   * appended to the document before the new one,
   * otherwise will fail to read 'bottom' value
   * in method popupManager.push
    */
  setTimeout(() => popupManager.push(ins), 10);
  return ins;
}

const Toast = function (options) {
  if (typeof options === 'string') {
    options = { msg: options };
  }
  
  if (!options.msg || typeof options.msg !== 'string') {
    throw new Error(`toast 参数有误--${JSON.stringify(options)}`);
  }
  
  const instance = new ToastCtr({
    propsData: options
  });
  
  return mountIns(instance).vm;
};

export default Toast;
